;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Burt Bro, by dahnamics
;;
;; Description: Similar to the Burt Bros. from Yoshi's Island and Yoshi's Island DS.  
;;		Jumps around and speeds up as he is hit.  The gfx change which show 
;;		that he loses his pants as he is hit.  Check the README for important
;;		information.
;;
;; Note:	Put him in an enclosed room with both walls and a ceiling. When he dies he 
;;		turns red/pink and bounces around the room.  He explodes on impact.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shy Guy, by mikeyk
;;
;; Description: A Shy Guy with many available configurations.
;;
;; Note: When rideable, clipping tables values should be: 03 0A FE 0E
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Uses first extra bit: NO
;;
;; Extra Property Byte 1
;;    bit 0 - enable spin killing
;;    bit 1 - stay on ledges
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	EXTRA_PROP_1 = $7FAB28
	RAM_ThrowTimer = $1504
	Invuln_Timer = $1564		; The RAM address for Larry's stun duration
	BounceHealth = $1528				; This is the HP of the rock. 8 bounces kills it.
	SPRITE_STATE    = $C2
	HIT_POINTS = $03        	; being stomped by Mario

	KILLED_XSPEED			dcb $30,$D0
	KILLED_YSPEED			dcb $30,$D0
	X_SPEED				dcb $08,$F8,$20,$E0,$38,$C8
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "INIT"
	LDA $167A,x
	STA $1528,x
	LDA #$40
	STA $1504,x
	LDA #$01		
	STA $151C,x                    

		PHY
                    JSR SUB_HORZ_POS
                    TYA
                    STA $157C,x
                    PLY

                    LDA $1588,x             ; if on the ground, reset the turn counter
                    ORA #$04
                    STA $1588,x             ; if on the ground, reset the turn counter

		STZ BounceHealth,x	; Set HP to 0, just in case it needs to be

                    RTL                 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"                                    
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR SPRITE_CODE_START   ;  |
                    PLB                     ;  |
                    RTL                     ; /



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN              RTS
SPRITE_CODE_START   JSR SUB_GFX       ; graphics routine

                    LDA $14C8,x             ; return if sprite status != 8

                    CMP #$08                ; \ if status != 8, return
                    BNE RETURN              ; /

                    LDA $9D                 ; \ if sprites locked, return
                    BNE RETURN              ; /

                    JSR SUB_OFF_SCREEN_X3   ; handle off screen situation

DONE_WITH_SPEED     JSL $018032             ; interact with sprites

		JSL $01A7DC		; interact with mario 
			BCC NO_CONTACT          ; return if no contact
			LDA $154C,x             ; \ if sprite invincibility timer > 0 ...
			BNE NO_CONTACT          ; /    ... goto NO_CONTACT
			LDA #$08                ; \ sprite invincibility timer = $08
			STA $154C,x             ; /
			LDA $7D                 ; \  if Mario's y speed < 10 ...
			CMP #$10                ;  }   ... sprite will hurt Mario
			BMI SPRITE_WINS         ; /    

MARIO_WINS		JSR SUB_STOMP_PTS       ; give Mario points
			JSL $01AA33             ; set Mario speed
			JSL $01AB99             ; display contact graphic
			LDA #$28                ; \ sound effect
			STA $1DFC               ; /
;			LDA #$01		    ; \ Set stunned state
;			STA $C2,x               ; /
			LDA #$10		    ; \ Set stunned timer
			STA $1564,x             ; /
			INC $1534,x             ; increment sprite hit counter
			LDA $1534,x             ; \ if sprite hit counter == 3
			CMP #HIT_POINTS         ;  |   
			BNE SMUSH_SPRITE        ; /

			LDA #$04                ; \ write frame to show
			STA $1602,x             ; / 
;			LDA #$02                ; \ set sprite status   
;			STA $14C8,x             ; /
;			STZ $B6,x               ; no x speed
;			LDA #$10                ; \ set y speed
;			STA $AA,x               ; /
			RTS                     ; return 

SMUSH_SPRITE		LDA #$0C                ; \ time to show semi-squashed sprite = $0C
			STA $1FE2,x             ; /
			RTS                     ; return

SPRITE_WINS         
	LDA $1497               ; \ if Mario is invincible...
	ORA $187A               ;  }  ... or Mario on yoshi...
	BNE NO_CONTACT          ; /   ... return
	JSR SUB_HORZ_POS        ; \  set new sprite direction
	TYA                     ;  }  
	STA $157C,x             ; /
	JSL $00F5B7             ; hurt Mario

;RETURN_24           RTS                     ; final return

NO_CONTACT          	LDA SPRITE_STATE,x     
			CMP #$01
			BEQ BLUSHING

;-----------------------------------------------------------------------------------------
; state 0
;-----------------------------------------------------------------------------------------

WALKING

		LDA $1534,x
		CMP #$03
		BNE CONTINUE
		LDA #$01		
		STA SPRITE_STATE,x	
		
				
CONTINUE
                    LDA $1588,x             ; \ if sprite is in contact with an object...
                    AND #$03                ;  |
                    BEQ NO_OBJ_CONTACT      ;  |
                    LDA $157C,x             ;  | flip the direction status
                    EOR #$01                ;  |
                    STA $157C,x             ; /

NO_OBJ_CONTACT      
                    LDA $1588,x             ; run the subroutine if the sprite is in the air...
                    ORA $151C,x             ; ...and not already turning
                    BNE X_TIME	           ;
                    JSR SUB_CHANGE_DIR      ;
                    LDA #$01                ; set that we're already turning
                    STA $151C,x             ;
                                        
X_TIME		LDA $1534,x             ; \ set x speed based on total HP
		ASL
		CLC
		ADC $157C,x		; and direction
		TAY

		LDA X_SPEED,y           ;  |
		STA $B6,x               ; /

		JSR Hop		; Jump to custom code

IN_AIR              JSL $01802A             ; update position based on speed values

Hop:
        LDA #$40		; If timer isn't 40..
        CMP $1504,x		; 
        BNE IncreaseHop	; Increase it.
        LDA $1588,x		; Don't jump if already on ground.
        AND #$04		; 
        BEQ RETURN0	; 
        LDA #$C0		; Set jump height.
        STA $AA,x		; Store it too. 
        STZ $1504,x		; Reset timer.
        RTS		; Return
IncreaseHop:
        INC $1504,x		; Increase timer.

RETURN0		RTS	

;-----------------------------------------------------------------------------------------
; state 1
;-----------------------------------------------------------------------------------------

BLUSHING

		LDA #$1F                ; \ sound effect
		STA $1DF9               ; /

		LDA $1588,x		; \ if sprite is in contact with an object...
		AND #$0C		;  |
		BEQ NO_CONTACT_Y		;  |
		LDA #$04            	; Kill Lololo or Lalala
		STA $14C8,x
		JSL $07FC3B             ; show star animation
		LDA #$08                ; \ play sound effect
		STA $1DF9               ; /


NO_CONTACT_Y		
			LDA $1588,x		; \ if sprite is in contact with an object...
			AND #$03		;  |
			BEQ NO_CONTACT_X		;  |
			LDA $157C,x		;  | flip the direction status
			EOR #$01		;  |
			STA $157C,x		; /


NO_CONTACT_X		LDY $151C,x		; \ set x speed based on direction
			LDA KILLED_YSPEED,y		;  |
			STA $AA,x		; /                    
			LDY $157C,x		; \ set x speed based on direction
			LDA KILLED_XSPEED,y		;  |
			STA $B6,x		; /
			
			                                        
IN_AIR2			JSL $01802A		; update position based on speed values
			JSL $018032			; Make the sprite interact with others. Make sure the .cfg setting for this
							; is off!

RETURN1		RTS	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;TILEMAP             dcb $46,$4A,$7F,$00
;                    dcb $46,$5A,$2F,$00
;                    dcb $48,$4A,$7F,$00
;                    dcb $48,$5A,$2F,$00

TILEMAP				

	dcb $00,$02,$20,$22		; no-hit 1: head left, head right, bottom left, bottom right
	dcb $04,$06,$24,$26		; no-hit 2: head left, head right, bottom left, bottom right
	dcb $08,$0A,$28,$2A		; one-hit 1: head left, head right, bottom left, bottom right
	dcb $0C,$0E,$2C,$2E		; one-hit 2: head left, head right, bottom left, bottom right
	dcb $40,$42,$60,$62		; two-hit 1: head left, head right, bottom left, bottom right
	dcb $44,$46,$64,$66		; two-hit 2: head left, head right, bottom left, bottom right
	dcb $48,$4A,$68,$6A		; dying : head left, head right, bottom left, bottom right
	dcb $4C,$4E,$6C,$6E		; dying2: head left, head right, bottom left, bottom right
					
	dcb $00,$02,$20,$22		; no-hit 1: head left, head right, bottom left, bottom right
	dcb $04,$06,$24,$26		; no-hit 2: head left, head right, bottom left, bottom right
	dcb $08,$0A,$28,$2A		; one-hit 1: head left, head right, bottom left, bottom right
	dcb $0C,$0E,$2C,$2E		; one-hit 2: head left, head right, bottom left, bottom right
	dcb $40,$42,$60,$62		; two-hit 1: head left, head right, bottom left, bottom right
	dcb $44,$46,$64,$66		; two-hit 2: head left, head right, bottom left, bottom right
	dcb $48,$4A,$68,$6A		; dying : head left, head right, bottom left, bottom right
	dcb $4C,$4E,$6C,$6E		; dying2: head left, head right, bottom left, bottom right
	dcb $4C,$4E,$6C,$6E		; dying2: head left, head right, bottom left, bottom right

HORZ_DISP           


	dcb $F8,$08,$F8,$08
	dcb $F8,$08,$F8,$08
	dcb $F8,$08,$F8,$08
	dcb $F8,$08,$F8,$08
	dcb $F8,$08,$F8,$08
	dcb $F8,$08,$F8,$08
	dcb $F8,$08,$F8,$08
	dcb $F8,$08,$F8,$08
					

	dcb $08,$F8,$08,$F8
	dcb $08,$F8,$08,$F8	
	dcb $08,$F8,$08,$F8
	dcb $08,$F8,$08,$F8
	dcb $08,$F8,$08,$F8
	dcb $08,$F8,$08,$F8
	dcb $08,$F8,$08,$F8
	dcb $08,$F8,$08,$F8
					
VERT_DISP           dcb $F0,$F0,$00,$00
					dcb $EF,$EF,$FF,$FF
					dcb $F0,$F0,$FF,$FF
					dcb $F0,$F0,$FF,$FF
					dcb $F0,$F0,$00,$00
					dcb $F0,$F0,$00,$00
					dcb $F0,$F0,$00,$00
					dcb $F0,$F0,$00,$00
					
					dcb $F0,$F0,$00,$00
					dcb $EF,$EF,$FF,$FF
					dcb $F0,$F0,$FF,$FF
					dcb $F0,$F0,$FF,$FF
					dcb $F0,$F0,$00,$00
					dcb $F0,$F0,$00,$00
					dcb $F0,$F0,$00,$00
					dcb $F0,$F0,$00,$00
					
PROPERTIES				dcb $3F,$3F,$3F,$3F		; 00 = no flip
					dcb $3F,$3F,$3F,$3F		; 40 = horizontal flip
					dcb $3F,$3F,$3F,$3F		; 80 = vertical flip
					dcb $3F,$3F,$3F,$3F		; C0 = horizontal and vertical flip
					dcb $3F,$3F,$3F,$3F
					dcb $3F,$3F,$3F,$3F
					dcb $3F,$3F,$3F,$3F
					dcb $3F,$3F,$3F,$3F
					
					dcb $7F,$7F,$7F,$7F
					dcb $7F,$7F,$7F,$7F
					dcb $7F,$7F,$7F,$7F
					dcb $7F,$7F,$7F,$7F
					dcb $7F,$7F,$7F,$7F
					dcb $7F,$7F,$7F,$7F
					dcb $7F,$7F,$7F,$7F
					dcb $7F,$7F,$7F,$7F


SUB_GFX:
		JSR GET_DRAW_INFO		; after: Y = index to sprite tile map ($300)
                                ;      $00 = sprite x position relative to screen boarder 
                                ;      $01 = sprite y position relative to screen boarder  
       
        LDA $1602,x
        STA $03                 ; | $03 = index to frame start (0 or 4)                   
       
		LDA $14					; Frame Counter
		LSR A					; \
		LSR A					; | Change every 8 frames
		LSR A					; |
		;LSR A					; | 
		;LSR A					; |  
		LSR A					; /

		AND #$01				; 2 walking "frames"
		ASL A					; x2
		ASL A					; x4 (sprite is made up of 4 tiles)
		STA $03

EMBARRASED:
		LDA $1534,x		; If waiting
		CMP #$03
		BNE TWO_HITS
		LDA $03					; |
		CLC						; |
		ADC #$18				; | ...use stun frames.
		STA $03					; /
		BRA DONE_WALKING

TWO_HITS:
		LDA $1534,x		; If waiting
		CMP #$02
		BNE ONE_HIT
		LDA $03					; |
		CLC						; |
		ADC #$10				; | ...use stun frames.
		STA $03					; /
		BRA DONE_WALKING

ONE_HIT:
		LDA $1534,x		; If waiting
		CMP #$01
		BNE DONE_WALKING
		LDA $03					; |
		CLC						; |
		ADC #$08				; | ...use stun frames.
		STA $03					; /
;		BRA DONE_WALKING
        
DONE_WALKING:
        LDA $157C,x             ; \ $02 = sprite direction
        STA $02                 ; /
        BNE LOOP_START
        LDA $03
        CLC
        ADC #$20 				; skip some tiles if facing the other way
        STA $03

LOOP_START:
		PHX                     ; push sprite index
        LDX #$03                ; loop counter = (number of tiles per frame) - 1

LOOP_START_2:
        PHX						; push current tile number
        ;TXA                    ; \ X = index to horizontal displacement
        ;ORA $03                ; / get index of tile (index to first tile of frame + current tile number)
        ;TAX
        
        TXA                     ; \ X = index of frame start + current tile
        CLC                     ;  |
        ADC $03                 ;  |
        TAX                     ; /
        
        PHY                     ; \
        TYA                     ; |
        LSR A                   ; |
        LSR A                   ; |
        TAY                     ; | Set tile to be 16x16 ($02)
        LDA #$02				; |
        STA $0460,y             ; |
        PLY                     ; /
        
        LDA $00                 ; \ tile x position = sprite x location ($00)
        CLC                     ;  |
        ADC HORZ_DISP,x			;  |
        STA $0300,y             ; /
        
        LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
        CLC                     ;  |
        ADC VERT_DISP,x			;  |
        STA $0301,y             ; /
        
        LDA TILEMAP,x           ; \ store tile
        STA $0302,y             ; / 
        
        PHX
        LDX $15E9
        LDA $15F6,x             ; get palette info
        PLX
        ORA PROPERTIES,x		; flip tile if necessary
        ORA $64                 ; add in tile priority of level
        STA $0303,y             ; store tile properties
        
        PLX                     ; \ pull, X = current tile of the frame we're drawing
        INY                     ;  | increase index to sprite tile map ($300)...
        INY                     ;  |    ...we wrote 1 16x16 tile...
        INY                     ;  |    ...sprite OAM is 8x8...
        INY                     ;  |    ...so increment 4 times
        DEX                     ;  | go to next tile of frame and loop
        BPL LOOP_START_2        ; / 

        PLX                     ; pull, X = sprite index
        LDY #$FF                ; \ We've already set 460, so use FF!
        LDA #$03                ; | A = number of tiles drawn - 1
        JSL $01B7B3             ; / don't draw if offscreen
        RTS                     ; return 

                    

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_CHANGE_DIR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $019098

SUB_CHANGE_DIR      ;LDA $15AC,x
                    ;BNE LABEL41
                    ;LDA #$08   
                    ;STA $15AC,x
                    LDA $B6,x  
                    EOR #$FF   
                    INC A      
                    STA $B6,x  
                    LDA $157C,x
                    EOR #$01   
                    STA $157C,x
LABEL41             RTS        



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

STAR_SOUNDS         dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS       PHY                     ; 
                    LDA $1697               ; \
                    CLC                     ;  | 
                    ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
                    INC $1697               ; increase consecutive enemies stomped
                    TAY                     ;
                    INY                     ;
                    CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
                    BCS NO_SOUND            ; /    ... don't play sound 
                    LDA STAR_SOUNDS,y       ; \ play sound effect
                    STA $1DF9               ; /   
NO_SOUND            TYA                     ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET            ;  |
                    LDA #$08                ; /
NO_RESET            JSL $02ACE5             ; give mario points
                    PLY                     ;
                    RTS                     ; return
                
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B829 - vertical mario/sprite position check - shared
; Y = 1 if mario below sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B829 

SUB_VERT_POS        LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
                    LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
                    SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
                    SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
                    STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
                    LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
                    SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
                    BPL LABEL11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
                    INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
LABEL11             RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B83B             

TABLE3              dcb $40,$B0
TABLE6              dcb $01,$FF 
TABLE4              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X0   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | 
SUB_OFF_SCREEN_X1   LDA #$04                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X2   LDA #$02                ;  |
STORE_03            STA $03                 ;  |
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_X3   STZ $03                 ; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_2            ; /    
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;A:0101 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0256 VC:090 00 FL:16953
                    CLC                     ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0280 VC:090 00 FL:16953
                    ADC TABLE4,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0294 VC:090 00 FL:16953
                    ROL $00                 ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0326 VC:090 00 FL:16953
                    CMP $E4,x               ;x pos ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0364 VC:090 00 FL:16953
                    PHP                     ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0394 VC:090 00 FL:16953
                    LDA $1B                 ;x boundry hi ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizCHC:0416 VC:090 00 FL:16953
                    LSR $00                 ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0440 VC:090 00 FL:16953
                    ADC TABLE5,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdizcHC:0478 VC:090 00 FL:16953
                    PLP                     ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizcHC:0510 VC:090 00 FL:16953
                    SBC $14E0,x             ;x pos high ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0538 VC:090 00 FL:16953
                    STA $00                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0570 VC:090 00 FL:16953
                    LSR $01                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0594 VC:090 00 FL:16953
                    BCC LABEL20             ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0632 VC:090 00 FL:16953
                    EOR #$80                ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0648 VC:090 00 FL:16953
                    STA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0664 VC:090 00 FL:16953
LABEL20             LDA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0688 VC:090 00 FL:16953
                    BPL RETURN_2            ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0712 VC:090 00 FL:16953
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KILL_SPRITE         ; /
                    LDY $161A,x             ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0140 VC:071 00 FL:21152
                    CPY #$FF                ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0172 VC:071 00 FL:21152
                    BEQ KILL_SPRITE         ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0188 VC:071 00 FL:21152
                    LDA #$00                ; \ mark sprite to come back    A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0204 VC:071 00 FL:21152
                    STA $1938,y             ; /                             A:FF00 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0220 VC:071 00 FL:21152
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_2            RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS RETURN_2            ; /
                    AND #$01                ;A:0227 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0228 VC:112 00 FL:1142
                    STA $01                 ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0244 VC:112 00 FL:1142
                    TAY                     ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0268 VC:112 00 FL:1142
                    LDA $1C                 ;A:0201 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0282 VC:112 00 FL:1142
                    CLC                     ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0306 VC:112 00 FL:1142
                    ADC TABLE3,y            ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0320 VC:112 00 FL:1142
                    ROL $00                 ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0352 VC:112 00 FL:1142
                    CMP $D8,x               ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0390 VC:112 00 FL:1142
                    PHP                     ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0420 VC:112 00 FL:1142
                    LDA.W $001D             ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNVMXdizcHC:0442 VC:112 00 FL:1142
                    LSR $00                 ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdiZcHC:0474 VC:112 00 FL:1142
                    ADC TABLE6,y            ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdizCHC:0512 VC:112 00 FL:1142
                    PLP                     ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0544 VC:112 00 FL:1142
                    SBC $14D4,x             ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0572 VC:112 00 FL:1142
                    STA $00                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0604 VC:112 00 FL:1142
                    LDY $01                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0628 VC:112 00 FL:1142
                    BEQ LABEL22             ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0652 VC:112 00 FL:1142
                    EOR #$80                ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0668 VC:112 00 FL:1142
                    STA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0684 VC:112 00 FL:1142
LABEL22             LDA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0708 VC:112 00 FL:1142
                    BPL RETURN_2            ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0732 VC:112 00 FL:1142
                    BMI ERASE_SPRITE        ;A:0280 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0170 VC:064 00 FL:1195

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return
